/*
    Scanbot SDK Flutter Plugin

    Copyright (c) 2019 doo GmbH. All rights reserved.
 */
package io.scanbot.sdk.flutter

import android.app.Activity
import android.app.Application
import android.content.Context
import android.net.Uri
import android.util.Log
import androidx.annotation.NonNull
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.JsonObject
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.scanbot.sap.IScanbotSDKLicenseErrorHandler
import io.scanbot.sap.SdkFeature
import io.scanbot.sap.Status
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.ScanbotSDKInitializer
import io.scanbot.sdk.core.contourdetector.ContourDetector
import io.scanbot.sdk.flutter.utils.StorageUtils
import io.scanbot.sdk.persistence.CameraImageFormat
import io.scanbot.sdk.persistence.PageStorageSettings
import kotlinx.coroutines.*
import java.io.File
import kotlin.coroutines.CoroutineContext


class ScanbotSdkPlugin : FlutterPlugin,
        ActivityAware,
        MethodCallHandler,
        CoroutineResultHandler(),
        CoroutineScope,
        ISdkSubpluginManager by SdkSubpluginManager() {

    private var job: Job = Job()
    private lateinit var scanbotSDK: ScanbotSDK
    private var channel: MethodChannel? = null
    private val gson = GsonBuilder()
            .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
            .create()
    private var currentActivity: Activity? = null
    private var appContext: Context? = null
    override val coroutineContext: CoroutineContext
        get() = job + Dispatchers.Default

    override fun onMethodCall(call: MethodCall, result: Result) {
        launch(Dispatchers.IO) {
            when (call.method) {
                "initScanbotSdk" -> {
                    appContext?.let { initScanbotSdk(it, call, result) }
                }
                "getLicenseStatus" -> {
                    isLicenseValid(call, result)
                }
                else -> {
                    if (!callMethod(call, result)) {
                        withContext(Dispatchers.Main) { result.notImplemented() }
                    }
                }
            }
        }
    }

    private suspend fun initScanbotSdk(
            context: Context,
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Main) {
            try {
                val licenseKey = call.argument<String>("licenseKey") ?: ""
                val scanFormat = call.argument<String>("imageFormat") ?: ""
                val storageBaseDirectory = call.argument<String>("storageBaseDirectory")
                val imageQuality = call.argument<Int>("imageQuality") ?: 80
                val previewTargetMax = call.argument<Int>("previewTargetMax") ?: 900
                val loggingEnabled = call.argument<Boolean>("loggingEnabled") ?: false
                val documentDetectorMode = call.argument<String>("documentDetectorMode")?.let {
                    ContourDetector.Type.valueOf(it)
                }
                val imageFormat = if (scanFormat.contains("PNG")) {
                    CameraImageFormat.PNG
                } else {
                    CameraImageFormat.JPG
                }

                val scanbotSDKInitializer = ScanbotSDKInitializer()

                if (!storageBaseDirectory.isNullOrEmpty()) {
                    val uri = Uri.parse(storageBaseDirectory)
                    val customStorageBaseDirectory = File(uri.path)
                    customStorageBaseDirectory.mkdirs()
                    if (customStorageBaseDirectory.isDirectory.not()) {
                        methodResult.error(
                                "IOException",
                                "Specified storageBaseDirectory is not valid: $uri",
                                null
                        )
                    }
                    if (!customStorageBaseDirectory.exists()) {
                        customStorageBaseDirectory.mkdirs()
                    }
                    scanbotSDKInitializer.sdkFilesDirectory(
                            context as Application,
                            customStorageBaseDirectory
                    )
                    StorageUtils.setCustomStorageBaseDirectory(customStorageBaseDirectory)
                    Log.d(
                            "SDK",
                            "Using custom storage base directory: " + customStorageBaseDirectory.absolutePath
                    )
                }
                documentDetectorMode?.let {
                    scanbotSDKInitializer.contourDetectorType(it)
                }
                scanbotSDKInitializer
                        .licenceErrorHandler(IScanbotSDKLicenseErrorHandler { status, feature ->
                            val error = gson.toJson(FlutterPluginLicenseError(status, feature))
                            channel?.invokeMethod("onLicenseError", error)
                        })
                        .withLogging(loggingEnabled)
                        .usePageStorageSettings(
                                PageStorageSettings.Builder()
                                        .imageFormat(imageFormat)
                                        .imageQuality(imageQuality)
                                        .previewTargetMax(previewTargetMax)
                                        .build()
                        )
                        .prepareOCRLanguagesBlobs(true)
                        .prepareMRZBlobs(true)
                        .preparePayFormBlobs(true)
                        .prepareBarcodeScannerBlobs(true)
                        .prepareDcBlobs(true)
                        .license(context as Application, licenseKey)
                        .initialize(context)
                scanbotSDK = ScanbotSDK(context)

                clearPlugins()

                currentActivity?.let {
                    attachActivityAwareSubplugins(it)
                }

                attachContextAwareSubplugins(context)

                if ("" != licenseKey.trim() && "null" != licenseKey.toLowerCase()) {
                    methodResult.success("Scanbot SDK initialized.")
                } else {
                    methodResult.success(
                            "Scanbot SDK initialized. Trial mode activated. You can now test all features for 60 seconds."
                    )
                }
            } catch (e: Throwable) {
                Log.e("SDK", e.toString())
                methodResult.error(e::class.java.simpleName, e.message, null)
            }
        }
    }

    private fun attachActivityAwareSubplugins(activity: Activity) {
        attachSubplugin(
                ScanbotSdkUiPlugin(
                        activity,
                        this@ScanbotSdkPlugin,
                        scanbotSDK
                )
        )
    }

    private fun attachContextAwareSubplugins(context: Context) {
        attachSubplugin(
                PageFileStoragePlugin(
                        context,
                        this@ScanbotSdkPlugin,
                        scanbotSDK
                )
        )

        attachSubplugin(
                PageProcessorPlugin(
                        context,
                        this@ScanbotSdkPlugin,
                        scanbotSDK
                )
        )
    }

    private fun detachContextAwareSubplugins() {
        detachSubpluginByType(PageFileStoragePlugin::class.java)
        detachSubpluginByType(PageProcessorPlugin::class.java)
    }

    private fun detachActivityAwareSubplugins() {
        detachSubpluginByType(ScanbotSdkUiPlugin::class.java)
    }

    private suspend fun isLicenseValid(
            call: MethodCall,
            methodResult: Result
    ) {
        withContext(Dispatchers.Main) {
            try {
                if (::scanbotSDK.isInitialized) {
                    val json = JsonObject()
                    json.addProperty("isLicenseValid", scanbotSDK.isLicenseValid)
                    methodResult.success(json.toString())
                } else {
                    methodResult.error(
                            "SDKNotInitialisedException",
                            "need to initialise sdk with licence",
                            null
                    )
                }
            } catch (e: Throwable) {
                methodResult.error(e::class.java.simpleName, e.message, null)
            }
        }
    }

    override fun onDetachedFromActivity() {
        Log.d("ScanbotBarcodeSdkPlugin", "onDetachedFromActivity")
        currentActivity = null
        detachActivityAwareSubplugins()
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        val activity = binding.activity
        Log.d("ScanbotBarcodeSdkPlugin", "onReattachedToActivityForConfigChanges : ${activity.localClassName}")

        binding.addActivityResultListener(this)
        currentActivity = activity
        if (::scanbotSDK.isInitialized) {
            attachActivityAwareSubplugins(activity)
        }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        val activity = binding.activity
        Log.d("ScanbotBarcodeSdkPlugin", "onAttachedToActivity : ${activity.localClassName}")
        binding.addActivityResultListener(this)
        currentActivity = activity
        if (::scanbotSDK.isInitialized) {
            attachActivityAwareSubplugins(activity)
        }
    }

    override fun onDetachedFromActivityForConfigChanges() {
        Log.d("ScanbotBarcodeSdkPlugin", "onDetachedFromActivityForConfigChanges")
        currentActivity = null
        detachActivityAwareSubplugins()
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        appContext = flutterPluginBinding.applicationContext
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "scanbot_sdk")
        channel?.setMethodCallHandler(this);
        Log.d("ScanbotBarcodeSdkPlugin", "onAttachedToEngine")
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        job.cancel()
        clearPlugins()
        Log.d("ScanbotBarcodeSdkPlugin", "onDetachedFromEngine")
    }
}

class FlutterPluginLicenseError(val status: Status, val feature: SdkFeature)