import 'package:flutter/cupertino.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/common_data.dart';
import 'package:scanbot_sdk/sdk_utils.dart';

part 'mrz_scanning_data.g.dart';

/// This class describes machine readable zone rtu-ui component scanned result
@JsonSerializable(
    nullable: true,
    createFactory: true,
    includeIfNull: false,
    explicitToJson: true)
class MrzScanningResult extends Result {
  List<MRZField> fields;
  int validCheckDigitsCount;
  int checkDigitsCount;

  MrzScanningResult(
      this.fields, this.validCheckDigitsCount, this.checkDigitsCount);

  MrzScanningResult.error(String message) {
    this.operationResult = OperationResult.ERROR;
    this.message = message;
  }

  MrzScanningResult.canceled() {
    this.operationResult = OperationResult.CANCELED;
  }

  factory MrzScanningResult.fromJson(Map<String, dynamic> json) =>
      _$MrzScanningResultFromJson(json);

  Map<String, dynamic> toJson() => _$MrzScanningResultToJson(this);
}

/// This class describes machine readable zone one scanned field data
@JsonSerializable(nullable: true, createFactory: true, includeIfNull: false)
class MRZField {
  final String value;
  final double averageRecognitionConfidence;
  final MRZFieldName name;

  MRZField(this.value, this.averageRecognitionConfidence, this.name);

  factory MRZField.fromJson(Map<String, dynamic> json) =>
      _$MRZFieldFromJson(json);

  Map<String, dynamic> toJson() => _$MRZFieldToJson(this);
}

enum MRZFieldName {
  Unknown,
  DocumentCode,
  IssuingStateOrOrganization,
  DepartmentOfIssuance,
  FirstName,
  LastName,
  Nationality,
  DateOfBirth,
  Gender,
  DateOfExpiry,
  PersonalNumber,
  TravelDocType,
  Optional1,
  Optional2,
  DiscreetIssuingStateOrOrganization
}

/// This class describes machine readable zone rtu-ui component configuration
@JsonSerializable(nullable: true, includeIfNull: false, explicitToJson: true)
class MrzScannerConfiguration {
  /// Background color of the top toolbar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarBackgroundColor;

  /// Color of the titles of all buttons in the top toolbar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarButtonsColor;

  /// Background color outside of the finder window.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color cameraOverlayColor;

  /// Color of the finder window's outline.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color finderLineColor;

  /// Color of the text hint under the finder window.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color finderTextHintColor;
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color bottomButtonsActiveColor;
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color bottomButtonsInactiveColor;

  /// Title of the cancel button.
  String cancelButtonTitle;

  /// Title of the button that opens the screen where the user can allow
  /// the usage of the camera by the app.
  String enableCameraButtonTitle;

  /// Text that will be displayed when the app
  /// is not allowed to use the camera, prompting the user
  /// to enable the usage of the camera.
  String enableCameraExplanationText;

  /// Text hint shown under the finder window.
  String finderTextHint;

  /// Height of the finder window in pixels.
  int finderHeight;

  /// Width of the finder window in pixels.
  int finderWidth;

  /// Thickness of the finder window's outline.
  int finderLineWidth;

  /// Controls whether to play a beep sound after a successful detection.
  /// Default value is TRUE.
  bool successBeepEnabled;

  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool flashEnabled;
  String flashButtonTitle;

  /// Orientation lock mode of the camera: PORTRAIT or LANDSCAPE.
  /// By default the camera orientation is not locked.
  CameraOrientationMode orientationLockMode;

  bool cancelButtonHidden;

  MrzScannerConfiguration(
      {this.topBarBackgroundColor,
      this.topBarButtonsColor,
      this.cameraOverlayColor,
      this.finderLineColor,
      this.finderTextHintColor,
      this.bottomButtonsActiveColor,
      this.bottomButtonsInactiveColor,
      this.cancelButtonTitle,
      this.enableCameraButtonTitle,
      this.enableCameraExplanationText,
      this.finderTextHint,
      this.finderHeight,
      this.finderWidth,
      this.finderLineWidth,
      this.successBeepEnabled,
      this.flashEnabled,
      this.flashButtonTitle,
      this.orientationLockMode,
      this.cancelButtonHidden});

  factory MrzScannerConfiguration.fromJson(Map<String, dynamic> json) =>
      _$MrzScannerConfigurationFromJson(json);

  Map<String, dynamic> toJson() => _$MrzScannerConfigurationToJson(this);
}
