import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:scanbot_sdk/create_tiff_data.dart';
import 'package:scanbot_sdk/ocr_data.dart';
import 'package:scanbot_sdk/render_pdf_data.dart';
import 'package:scanbot_sdk/scanbot_sdk_models.dart';
import 'package:scanbot_sdk/sdk_utils.dart';

import 'common_data.dart';

class ScanbotSdk {
  static const MethodChannel _channel = const MethodChannel('scanbot_sdk');

  static Function(Status, SdkFeature) _sdkLicenseListener;

  /// Initializes the Scanbot SDK with given [config]s.
  /// The Scanbot SDK must be initialized before using any other API methods.
  static initScanbotSdk(ScanbotSdkConfig config) async {
    try {
      _sdkLicenseListener = config.licenseErrorHandler;
      _channel.setMethodCallHandler(_handler);
      var arguments = {};
      addParam(config.licenseKey, "licenseKey", arguments);
      addParam(imageFormatMap[config.imageFormat], "imageFormat", arguments);
      addParam(contourDetectorTypeMap[config.documentDetectorMode], "documentDetectorMode", arguments);
      addParam(config.imageQuality, "imageQuality", arguments);
      addParam(config.loggingEnabled, "loggingEnabled", arguments);
      addParam(config.storageBaseDirectory, "storageBaseDirectory", arguments);
      await _channel.invokeMethod("initScanbotSdk", arguments);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  static Future _handler(MethodCall call) {
    switch (call.method) {
      case "onLicenseError":
        var e = SdkLicenseException.fromJson(jsonDecode(call.arguments));
        if (_sdkLicenseListener != null) {
          _sdkLicenseListener(e.status, e.feature);
        }
        break;
    }
  }

  /// Performs auto document detection on the ORIGINAL image of the given [page]
  /// and creates (cropped) DOCUMENT image.
  /// The given [page] must have the ORIGINAL image.
  /// return [Page] with detected polygon.
  static Future<Page> detectDocument(Page page) async {
    try {
      var arguments = {"page": page.toJson()};
      String documentScanningResult =
          await _channel.invokeMethod("detectDocument", arguments);
      var json = jsonDecode(documentScanningResult);
      var pageResult = Page.fromJson(json);
      return pageResult;
    } catch (e) {
      return null;
    }
  }

  /// Returns the current license status of the Scanbot SDK.
  static Future<SdkLicenseStatus> getLicenseStatus() async {
    try {
      var licenseStatus = await _channel.invokeMethod("getLicenseStatus", {});
      var json = jsonDecode(licenseStatus);
      return SdkLicenseStatus.fromJson(json);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Renders a multi-page PDF document from DOCUMENT images of the given [pages]
  /// and stores it as a file. For each DOCUMENT image a separate PDF page is generated.
  /// All given [pages] must have the DOCUMENT images.
  ///
  /// [pages] list of pages to render
  /// [options] additional options of rendering
  /// return File [Uri] of the PDF document.
  static Future<Uri> createPdf(
      List<Page> pages, PdfRenderingOptions options) async {
    try {
      var pagesList = pages.map((page) => page.toJson()).toList();
      var arguments = {"pages": pagesList, "options": options.toJson()};
      final String uri = await _channel.invokeMethod("createPdf", arguments);
      return Uri.parse(uri);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Renders a multi-page TIFF image from DOCUMENT images of the given [pages]
  /// and stores it as a file. For each DOCUMENT image a separate TIFF page is generated.
  /// All given [pages] must have the DOCUMENT images.
  ///
  /// [pages] list of pages to render
  /// [options] additional options of rendering
  static Future<Uri> createTiff(
      List<Page> pages, TiffCreationOptions options) async {
    try {
      if (!options.binarized &&
          PredefinedCompressionTypes.binarizedOnlyCompressionTypes()
              .contains(options.compression)) {
        throw ("Incompatible compression type. The given compression type ${options.compression} is only applicable to binarized images.");
      }

      var pagesList = pages.map((page) => page.toJson()).toList();
      var arguments = {"pages": pagesList, "options": options.toJson()};
      final String uri = await _channel.invokeMethod("createTiff", arguments);
      return Uri.parse(uri);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Cleanups the storage folders of the Scanbot SDK.
  /// By calling this function all output files generated by the native Scanbot SDKs
  /// as well as by this Plugin will be deleted, including the image files of all scanned
  /// or imported Page objects, all generated PDF files, TIFF files, etc.
  static cleanupStorage() async {
    try {
      await _channel.invokeMethod("cleanupStorage");
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Removes a page with all its files (ORIGINAL image, DOCUMENT image,
  /// previews/thumbnails, filtered images, etc) from the file storage of the Scanbot SDK.
  /// This method does not remove any exported files (PDF, TIFF, etc) which were
  /// generated based on this page object.
  static deletePage(Page page) async {
    try {
      var arguments = {"page": page.toJson()};
      await _channel.invokeMethod("deletePage", arguments);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Creates a page from an image file URI. The given image will be used as ORIGINAL image.
  /// Use the [shouldDetectDocument] flag to specify whether the auto-document detection
  /// should be performed and the (cropped) DOCUMENT image should be created.
  /// return [Page]
  static Future<Page> createPage(
      Uri originalImageFileUri, bool shouldDetectDocument) async {
    try {
      var arguments = {
        "originalImageFileUri": originalImageFileUri.toString(),
        "shouldDetectDocument": shouldDetectDocument
      };
      final String pageJson =
          await _channel.invokeMethod("createPage", arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Rotates all images of the given [page] by 90 degrees value for [rotationsCount] times.
  static Future<Page> rotatePageClockwise(Page page, int rotationsCount) async {
    try {
      var arguments = {"page": page.toJson(), "rotationsCount": rotationsCount};
      String pageJson =
          await _channel.invokeMethod("rotatePageClockwise", arguments);
      var decoded = jsonDecode(pageJson);
      var rotatedPage = Page.fromJson(decoded);
      return rotatedPage;
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Applies an image [filter] on the final DOCUMENT image of given [page].
  static Future<Page> applyImageFilter(
      Page page, ImageFilterType filter) async {
    try {
      var arguments = {
        "page": page.toJson(),
        "filter": imageFilterJsonValue(filter)
      };
      final String pageJson =
          await _channel.invokeMethod("applyImageFilter", arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Creates a preview image file of the DOCUMENT image by given [filter].
  /// Does not modify the actual DOCUMENT image of the passed [page].
  /// Use the "applyImageFilter(page, filter)" method to finally apply an image filter on a page.
  static Future<Uri> getFilteredDocumentPreviewUri(
      Page page, ImageFilterType filter) async {
    try {
      var arguments = {
        "page": page.toJson(),
        "filter": imageFilterJsonValue(filter)
      };
      final String uri = await _channel.invokeMethod(
          "getFilteredDocumentPreviewUri", arguments);
      return Uri.parse(uri);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Returns current OCR configs, like installed OCR languages, etc.
  static Future<SdkOcrConfig> getOcrConfigs() async {
    try {
      String result = await _channel.invokeMethod("getOcrConfig");
      var json = jsonDecode(result);
      return SdkOcrConfig.fromJson(json);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Performs OCR (Optical Character Recognition) on DOCUMENT images of given pages.
  static Future<OcrResult> performOcr(
      List<Page> pages, OcrOptions options) async {
    try {
      var arguments = {
        "pages": pages?.map((e) => e?.toJson())?.toList(),
        "options": options.toJson()
      };

      String result = await _channel.invokeMethod("performOcr", arguments);
      var json = jsonDecode(result);
      return OcrResult.fromJson(json);
    } catch (e) {
      print(e);
      throw e;
    }
  }
}
